﻿using System;
using PX.Data.Unit;
using PX.Objects.Unit;
using Autofac;
using Xunit;
using PX.Data;
using PX.Objects.CR;
using PX.Objects.GL;
using PX.Objects.GL.FinPeriods;
using PX.Objects.AP;
using PX.Objects.AR;

namespace TestSample
{
	public class TestEntry : TestBase
	{
		protected override void RegisterServices(ContainerBuilder builder)
		{
			base.RegisterServices(builder);
			builder.Register<Func<PXGraph, IFinPeriodRepository>>(context =>
			{
				return (graph) => new FinPeriodServiceMock();
			});
		}
		[Fact]
		public void ARInvoiceEntry()
		{
			Setup<ARInvoiceEntry>(
				new ARSetup { RequireControlTotal = false },
				new GLSetup { YtdNetIncAccountID = int.MaxValue });
			ARInvoiceEntry graph = PXGraph.CreateInstance<ARInvoiceEntry>();
			ARInvoice invoice = graph.Document.Insert();
			BAccountR baccount = (BAccountR)graph.Caches[typeof(BAccountR)].Insert(new BAccountR { AcctCD = "C1", Status = "A" });
			Location location = (Location)graph.Caches[typeof(Location)].Insert(new Location { BAccountID = baccount.BAccountID, LocationCD = "L01", IsActive = true });
			invoice.CustomerID = baccount.BAccountID;
			invoice.CustomerLocationID = location.LocationID;
			invoice = graph.Document.Update(invoice);
			ARTran tran = graph.Transactions.Insert();
			Assert.NotNull(tran);
			tran.CuryExtPrice = 10.0m;
			tran = graph.Transactions.Update(tran);
			Assert.Equal(tran.CuryExtPrice, 10.0m);
			Assert.Equal(invoice.CuryLineTotal, 10.0m);
		}
		[Fact]
		public void APBillEntry()
		{
			Setup<APInvoiceEntry>(
				new APSetup { },
				new GLSetup { YtdNetIncAccountID = int.MaxValue });
			APInvoiceEntry graph = PXGraph.CreateInstance<APInvoiceEntry>();
			APInvoice invoice = graph.Document.Insert();
			BAccountR baccount = (BAccountR)graph.Caches[typeof(BAccountR)].Insert(new BAccountR { AcctCD = "V01", Status = "A" });
			Location location = (Location)graph.Caches[typeof(Location)].Insert(new Location { BAccountID = baccount.BAccountID, LocationCD = "L01", IsActive = true });
			Tail<APInvoice, APInvoice.vendorID>(graph, (row, query, pars) =>
					new PXResult<BAccountR, Vendor>(
						baccount,
						new Vendor { Status = baccount.Status }));
			invoice.VendorID = baccount.BAccountID;
			invoice.VendorLocationID = location.LocationID;
			invoice = graph.Document.Update(invoice);
			APTran tran = graph.Transactions.Insert();
			Assert.NotNull(tran);
			tran.CuryLineAmt = 10.0m;
			tran = graph.Transactions.Update(tran);
			Assert.Equal(tran.CuryLineAmt, 10.0m);
			Assert.Equal(invoice.CuryLineTotal, 10.0m);
		}
		[Fact]
		public void JourrnalEntry()
		{
			Setup<JournalEntry>(
				new GLSetup { YtdNetIncAccountID = int.MaxValue, RequireControlTotal = false });
			JournalEntry graph = PXGraph.CreateInstance<JournalEntry>();
			Batch batch = graph.BatchModule.Insert();
			GLTran tran = graph.GLTranModuleBatNbr.Insert();
			tran.CuryCreditAmt = 10.0m;
			tran = graph.GLTranModuleBatNbr.Update(tran);
			GLTran tran2 = graph.GLTranModuleBatNbr.Insert();
			Assert.Equal(tran.CuryCreditAmt, tran2.CuryDebitAmt);
			Assert.Equal(batch.CuryCreditTotal, batch.CuryDebitTotal);
			Assert.Equal(batch.CuryControlTotal, 10.0m);
		}
	}
}