﻿using System;
using PX.Data.Unit;
using PX.Objects.Unit;
using Autofac;
using Xunit;
using PX.Data;
using PX.Objects.GL;
using PX.Objects.GL.FinPeriods;
using PX.Objects.AP;
using PX.Objects.PO;
using PX.Objects.TX;

namespace TestSample
{
	public class TestProcess : TestBase
	{
		protected override void RegisterServices(ContainerBuilder builder)
		{
			base.RegisterServices(builder);
			builder.Register<Func<PXGraph, IFinPeriodRepository>>(context =>
			{
				return (graph) => new FinPeriodServiceMock();
			});
		}
		public class APReleaseProcessMock : APReleaseProcess
		{
			public virtual decimal PublicGetLandedCostAmount(POLandedCostDetail landedCostDetail)
			{
				return GetLandedCostAmount(landedCostDetail);
			}
		}
		[Fact]
		public void GetLandedCostAmount()
		{
			Setup<APReleaseProcessMock>(new APSetup { }, new GLSetup { });
			APReleaseProcessMock graph = PXGraph.CreateInstance<APReleaseProcessMock>();
			POLandedCostDetail lcdet = graph.landedCostDetails
				.Insert(new POLandedCostDetail { DocType = POLandedCostDocType.LandedCost, RefNbr = "LC1", LineNbr = 1, CuryLineAmt = 10m });
			POLandedCostTax lctax = (POLandedCostTax)graph.Caches[typeof(POLandedCostTax)]
				.Insert(new POLandedCostTax { DocType = lcdet.DocType, RefNbr = lcdet.RefNbr, LineNbr = lcdet.LineNbr, CuryTaxableAmt = lcdet.CuryLineAmt - 1m });
			decimal lcamt = graph.PublicGetLandedCostAmount(lcdet);
			Assert.True(lcamt == lcdet.CuryLineAmt);
			Tail<POLandedCostTax>(graph,
				(row, query, pars) => new PXResult<POLandedCostTax, Tax>(row, new Tax { TaxCalcLevel = CSTaxCalcLevel.Inclusive }));
			lcamt = graph.PublicGetLandedCostAmount(lcdet);
			Assert.True(lcamt == lctax.CuryTaxableAmt);
		}
	}
}